﻿Imports System.Text
Imports Telerik.WinControls.UI
Imports Telerik.WinControls
Imports System.ComponentModel
Imports Telerik.WinControls.Data
Imports System.Runtime.InteropServices

Public Delegate Sub ChangedEventHandler(ByVal sender As Object, ByVal e As ItemsChangedEventArgs)

Public Class RadMultiSelectDropDownList
    Inherits RadDropDownList

    <DllImport("user32")> _
    Private Shared Function HideCaret(ByVal hwnd As IntPtr) As Boolean
    End Function

    Private selectedItems_Renamed As MyDynamicCollection
    Public Event SelectedItemsChanged As ChangedEventHandler

    Public Sub New()
        selectedItems_Renamed = New MyDynamicCollection(Me)
        WireEvents()
    End Sub

    Private Sub WireEvents()
        AddHandler ListElement.ItemsChanged, AddressOf ListElement_ItemsChanged
        AddHandler ListElement.ItemsChanging, AddressOf ListElement_ItemsChanging

        AddHandler DropDownListElement.TextBox.TextBoxItem.HostedControl.GotFocus, AddressOf HostedControl_GotFocus
    End Sub

    Private Sub HostedControl_GotFocus(ByVal sender As Object, ByVal e As EventArgs)
        HideCaret(DirectCast(DropDownListElement.TextBox.TextBoxItem.HostedControl, TextBox).Handle)
    End Sub

    Private Sub ListElement_ItemsChanging(ByVal sender As Object, ByVal e As NotifyCollectionChangingEventArgs)
        If e.Action = NotifyCollectionChangedAction.Reset Then
            For i As Integer = 0 To Me.Items.Count - 1
                RemoveHandler (CType(Items(i), CustomListDataItem)).Changed, AddressOf OnSelectedItemChanged
            Next i
        End If
    End Sub

    Private Sub ListElement_ItemsChanged(ByVal sender As Object, ByVal e As NotifyCollectionChangedEventArgs)
        If e.Action = NotifyCollectionChangedAction.Add Then
            AddHandler (CType(e.NewItems(0), CustomListDataItem)).Changed, AddressOf OnSelectedItemChanged
        End If

        If e.Action = NotifyCollectionChangedAction.Remove Then
            RemoveHandler (CType(e.OldItems(0), CustomListDataItem)).Changed, AddressOf OnSelectedItemChanged
        End If
    End Sub

    Protected Overridable Sub OnSelectedItemChanged(ByVal sender As Object, ByVal e As ItemsChangedEventArgs)
        RaiseEvent SelectedItemsChanged(Me, e)
    End Sub

    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        Me.UnwireEvents()
        MyBase.Dispose(disposing)
    End Sub

    Private Sub UnwireEvents()
        RemoveHandler ListElement.ItemsChanged, AddressOf ListElement_ItemsChanged
        RemoveHandler ListElement.ItemsChanging, AddressOf ListElement_ItemsChanging

        RemoveHandler DropDownListElement.TextBox.TextBoxItem.HostedControl.GotFocus, AddressOf HostedControl_GotFocus
    End Sub

    Public Overloads ReadOnly Property SelectedItems() As MyDynamicCollection
        Get
            Return selectedItems_Renamed
        End Get
    End Property

    Protected Overrides Sub OnCreatingVisualItem(ByVal sender As Object, ByVal args As CreatingVisualListItemEventArgs)
        MyBase.OnCreatingVisualItem(sender, args)

        args.VisualItem = New CustomListVisualItem()
    End Sub

    Protected Overrides Sub OnSelectedIndexChanged(ByVal sender As Object, ByVal newIndex As Integer)
        MyBase.OnSelectedIndexChanged(sender, newIndex)

        SetText()
    End Sub

    Friend Sub SetText()
        Me.BeginUpdate()
        Me.Text = String.Empty
        For Each item As CustomListDataItem In Me.Items
            If item.Checked Then
                Me.DropDownListElement.EditableElementText += item.Text & "; "
            End If
        Next item
        Me.EndUpdate()
    End Sub
End Class

